global g


### Utility functions


proc util_errbox { s } { tk_dialog .utilbox Error $s "" 0 OK }


proc util_tipbox { name s } {
   global g

   if { [ regexp Unregistered [ wbSec ] ] } {
      set fullname state_tip_$name
      if { ! $g($fullname) } {
         tk_dialog .utilbox "Quick Tip" $s "" 0 OK
         set g($fullname) 1 } } }


proc util_tipbox_le { } { util_tipbox landed "Use Climate->Compute to finish" }


proc util_busy_one { win state } {
   global g

   if { [ string compare $state on ] } {
      $win config -cursor $g(current_cursor)
      set g(working_message) ""
   } else {
      set g(current_cursor) [lindex [. config -cursor] 4]
      $win config -cursor watch
      set g(working_message) "Working..."
   } }


# was wrapper for tixBusy, so win argument is not used
proc util_busy { win state } {
   global g

   if { $g(videt_showing)   } { util_busy_one .videt_dlg   $state }
   if { $g(climpar_showing) } { util_busy_one .climpar_dlg $state }
   if { $g(viglobe_showing) } { util_busy_one .viglobe_dlg $state }
   util_busy_one . $state }


proc util_flags { args } {
   global g

   set len [ llength $args ]
   for { set i 0 } { $i < $len } { incr i 2 } {
      set name [ lindex $args $i ]
      set value [ lindex $args [ expr $i + 1 ] ]
      set g($name) $value }
   wbFlags $g(state_have_rivers) $g(viglobe_render) $g(state_season) \
      $g(climpar_interpol) $g(state_clim_valid) }


proc util_click { x y } {
   global g

   set x [ .main canvasx $x ]
   set y [ .main canvasy $y ]
   set scale [ expr $g(current_zoom) * 5 ]
   set x [ expr $x / $scale ]
   set y [ expr $y / $scale ]
   if { $x > 89 } { set x 89 }
   if { $x < 0  } { set x 0  }
   if { $y > 89 } { set y 89 }
   if { $y < 0  } { set y 0  }
   if { $g(videt_showing) } {
      videt_click $x $y
   } elseif { $g(region_valid) } {
      region_click $x $y } }


### File menu commands


proc file_open { } {
   global g

   set types  {{{WB files} {.wb}}}
   set filename [ tk_getOpenFile -filetypes $types -title "Open File" ]
   if { [ string compare $filename "" ] } {
      util_busy . on
      update idletasks
      region_freeall
      videt_close
      climpar_close
      viglobe_close
      set ret [ wbLoad $filename ]
      set l0 [ lindex $ret 0 ]
      set g(state_have_blob)      $l0
      set g(state_land_anchored) $l0
      util_flags state_clim_valid [ lindex $ret 1 ] \
         state_have_rivers [ lindex $ret 2 ]
      if { $g(state_clim_valid) } {
         view_facet clim
      } else {
         view_facet tec
      }
      util_busy . off } }


proc file_save { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before saving"
   } else {
      set types  {{{WB files} {.wb}}}
      set filename [ tk_getSaveFile -filetypes $types -title "Save File" ]
      if { [ string compare $filename "" ] } {
         util_busy . on
         update idletasks
         wbSave $filename $g(state_clim_valid) $g(state_have_rivers)
         util_busy . off } } }


proc file_export_gif { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before exporting GIF"
   } else {
      set types  {{{GIF files} {.gif}}}
      set filename [ tk_getSaveFile -filetypes $types -title "Export GIF" ]
      if { [ string compare $filename "" ] } {
         if { [ regexp Unregistered [ wbSec ] ] } {
            util_errbox "Save only allowed in registered version"
         } else {
            util_busy . on
            update idletasks
            main_photo write $filename -format gif
            util_busy . off } } } }


proc file_export_land { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before exporting land"
   } else {
      set types  {{{GIF files} {.gif}}}
      set filename [ tk_getSaveFile -filetypes $types -title "Export Land" ]
      if { [ string compare $filename "" ] } {
         util_busy . on
         update idletasks
         set j 0
         set ra [ wbGetland ]
         image create photo temp_photo -width 90 -height 90
         foreach l $ra {
            set i 0
            foreach z $l {
               set col [ format "#%02x%02x%02x" $z $z $z ]
               temp_photo put $col -to $i $j
               incr i }
            incr j }
         temp_photo write $filename -format gif
         image delete temp_photo
         util_busy . off } } }


proc file_new { } {
   global g

   util_busy . on
   update idletasks
   wbNew
   set g(state_have_blob)       1
   set g(state_land_anchored)   0
   set g(state_clim_valid)      0
   util_flags state_have_rivers 0
   view_facet tec
   util_busy . off
   util_tipbox filenew "Use Land->Drift to move continents" }


### View menu commands


proc view_facet { name } {
   global g

   set g(current_facet) $name
   image create photo main_photo -format map -data $name
   .main config -scrollregion "0 0 450 450"
   set g(current_zoom) 1 }


proc view_land { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before viewing land"
   } elseif { [ string compare $g(current_facet) tec ] } {
      util_busy . on
      update idletasks
      view_facet tec
      util_busy . off } }


proc view_clim { name } {
   global g

   if { ! $g(state_clim_valid) } {
      util_errbox "Use Climate->Compute before viewing climate"
   } elseif { [ string compare $g(current_facet) $name ] } {
      util_busy . on
      update idletasks
      view_facet $name
      util_busy . off } }


proc view_zoomin { } {
   global g

   set level [ expr 2 * $g(current_zoom) ]
   set g(current_zoom) $level
   set size [ expr 450 * $level ]
   util_busy . on
   update idletasks
   image create photo temp_photo -width $size -height $size
   temp_photo copy main_photo -zoom 2
   main_photo copy temp_photo
   region_refresh
   image delete temp_photo
   .main config -scrollregion "0 0 $size $size"
   util_busy . off }


proc view_zoomout { } {
   global g

   if { $g(current_zoom) > 1 } {
      set level [ expr $g(current_zoom) / 2 ]
      set g(current_zoom) $level
      set size [ expr 450 * $level ]
      util_busy . on
      update idletasks
      image create photo temp_photo -width $size -height $size
      temp_photo copy main_photo -subsample 2
      main_photo copy temp_photo -shrink
      region_refresh
      image delete temp_photo
      .main config -scrollregion "0 0 $size $size"
      util_busy . off } }


proc viglobe_close { } {
   global g

   destroy .viglobe_dlg
   set g(viglobe_showing) 0 }


proc viglobe_ok { } {
   global g

   viglobe_close
   util_busy . on
   update idletasks
   set g(viglobe_spin) $g(viglobe_new_spin)
   util_flags viglobe_render $g(viglobe_new_render)
   wbGlobe $g(viglobe_spin)
   view_facet globe
   util_busy . off }


proc view_globe { } {
   global g

   if { ! $g(viglobe_showing) } {
      if { ! $g(state_have_blob) } {
         util_errbox "Use File->New before viewing globe"
      } elseif { ! [ string compare $g(current_facet) globe ] } {
         util_errbox "Cannot display globe when globe displayed"
      } else {
         set g(viglobe_showing) 1
         videt_close
         set g(viglobe_new_spin)   $g(viglobe_spin)
         set g(viglobe_new_render) $g(viglobe_render)
         toplevel .viglobe_dlg

         # Top level frame
         frame .viglobe_dlg.top
         frame .viglobe_dlg.btns
         grid .viglobe_dlg.top -column 0 -row 0
         grid .viglobe_dlg.btns -column 0 -row 1

         # Scalebar subframe
         frame .viglobe_dlg.top.pf
         scale .viglobe_dlg.top.pf.s -orient horizontal -label "" -showvalue 0 \
            -variable g(viglobe_new_spin) -to 63
         label .viglobe_dlg.top.pf.l -textvariable g(viglobe_new_spin)
         grid .viglobe_dlg.top.pf.s -column 0 -row 0
         grid .viglobe_dlg.top.pf.l -column 1 -row 0

         # Parameters
         label .viglobe_dlg.top.pl -text "Offset:" -justify right
         label .viglobe_dlg.top.ll -text "Rendered:" -justify right
         checkbutton .viglobe_dlg.top.lb -variable g(viglobe_new_render)
         grid .viglobe_dlg.top.pl -column 0 -row 0 -sticky e
         grid .viglobe_dlg.top.ll -column 0 -row 1 -sticky e
         grid .viglobe_dlg.top.pf -column 1 -row 0 -sticky w
         grid .viglobe_dlg.top.lb -column 1 -row 1 -sticky w

         # Buttons and spacers
         label .viglobe_dlg.btns.l
         label .viglobe_dlg.btns.r
         button .viglobe_dlg.btns.ok -text OK -command viglobe_ok
         button .viglobe_dlg.btns.can -text Cancel -command viglobe_close
         grid .viglobe_dlg.btns.l   -column 0 -row 0
         grid .viglobe_dlg.btns.ok  -column 1 -row 0
         grid .viglobe_dlg.btns.can -column 2 -row 0
         grid .viglobe_dlg.btns.r   -column 3 -row 0

         # Window manager stuff
         wm title .viglobe_dlg "View Globe"
         wm protocol .viglobe_dlg WM_DELETE_WINDOW viglobe_close } } }


### Land menu commands


proc land_n { n } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before drifting"
   } elseif { $g(state_land_anchored) } {
      util_errbox "May not drift after anchoring"
   } else {
      util_busy . on
      update idletasks
      wbDrift $n
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off
      util_tipbox drift "Use Land->Anchor when continents look interesting" } }


proc land_anchor { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before anchoring"
   } elseif { ! $g(state_land_anchored) } {
      util_tipbox anchor "Use Land->Edit menu to fine-tune continents"
      set g(state_land_anchored) 1 } }


proc land_flip { } {
   global g

   if { ! $g(state_land_anchored) } {
      util_errbox "Use Land->Anchor before flipping"
   } else {
      util_busy . on
      update idletasks
      wbFlip
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off
      util_tipbox_le } }


proc land_pan { dx dy } {
   global g

   if { ! $g(state_land_anchored) } {
      util_errbox "Use Land->Anchor before panning"
   } else {
      util_busy . on
      update idletasks
      wbPan $dx $dy
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off
      util_tipbox_le } }


proc land_sharpen { } {
   global g

   if { ! $g(state_land_anchored) } {
      util_errbox "Use Land->Anchor before sharpening"
   } else {
      util_busy . on
      update idletasks
      wbSharp
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off
      util_tipbox_le } }


proc land_crumble { } {
   global g

   if { ! $g(state_land_anchored) } {
      util_errbox "Use Land->Anchor before crumbling"
   } else {
      util_busy . on
      update idletasks
      wbCrumble
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off
      util_tipbox_le } }


proc land_undo { } {
   global g

   if { ! $g(state_land_anchored) } {
      util_errbox "Use Land->Anchor before undoing"
   } else {
      util_busy . on
      update idletasks
      wbUndo
      view_facet tec
      util_flags state_have_rivers 0 state_clim_valid 0
      util_busy . off } }


proc file_import_land { } {
   global g

   set types  {{{GIF files} {.gif}}}
   set filename [ tk_getOpenFile -filetypes $types -title "Import Land" ]
   if { [ string compare $filename "" ] } {
      util_busy . on
      update idletasks
      region_freeall
      videt_close
      climpar_close
      viglobe_close
      image create photo temp_photo -width 90 -height 90
      temp_photo read $filename -format gif
      set ra {}
      for { set j 0 } { $j < 90 } { incr j } {
         for { set i 0 } { $i < 90 } { incr i } {
            set rgb [ temp_photo get $i $j ]
            lappend ra [ lindex $rgb 0 ] } }
      image delete temp_photo
      wbPutland $ra
      set g(state_have_blob)       1
      set g(state_land_anchored)   1
      set g(state_clim_valid)      0
      util_flags state_have_rivers 0
      view_facet tec
      util_busy . off } }


### Climate menu commands


proc clim_compute { } {
   global g

   if { ! $g(state_have_blob) } {
      util_errbox "Use File->New before computing climate"
   } elseif { ! $g(state_clim_valid) } {
      util_flags state_have_rivers 0 state_clim_valid 1
      util_busy . on
      update idletasks
      wbClim
      view_facet clim
      util_busy . off } }


proc clim_river { } {
   global g

   if { ! $g(state_clim_valid) } {
      util_errbox "Use Climate->Compute before adding rivers"
   } else {
      util_busy . on
      update idletasks
      wbRiver
      util_flags state_have_rivers 1
      view_facet clim
      util_busy . off } }


### Help menu commands


proc help_close { } {
   global g

   destroy .help_dlg
   set g(help_showing) 0 }


proc help_about { } {
   global g

   if { ! $g(help_showing) } {
      set g(help_showing) 1
      toplevel .help_dlg
      label .help_dlg.t0 -text "World Builder Version 1.3"
      label .help_dlg.t1 -text "Copyright 1998 David L. Allen"
      label .help_dlg.t2 -text "http://www.best.com/~jendave/builder/world/"
      label .help_dlg.t3 -text [ wbSec ]
      label .help_dlg.l
      label .help_dlg.r
      button .help_dlg.ok -text OK -command help_close
      grid .help_dlg.t0 -column 0 -row 0 -columnspan 3
      grid .help_dlg.t1 -column 0 -row 1 -columnspan 3
      grid .help_dlg.t2 -column 0 -row 2 -columnspan 3
      grid .help_dlg.t3 -column 0 -row 3 -columnspan 3
      grid .help_dlg.l  -column 0 -row 4
      grid .help_dlg.ok -column 1 -row 4
      grid .help_dlg.r  -column 2 -row 4

      # Window manager stuff
      wm title .help_dlg "Help About"
      wm protocol .help_dlg WM_DELETE_WINDOW help_close } }


### Initialization


proc init_vars { } {
   global g

   set g(current_facet)         ""
   set g(current_zoom)           1
   set g(state_have_blob)        0
   set g(state_land_anchored)    0
   set g(state_clim_valid)       0
   set g(state_have_rivers)      0
   set g(state_season)          -1
   set g(state_tip_landed)       0
   set g(state_tip_filenew)      0
   set g(state_tip_drift)        0
   set g(state_tip_anchor)       0
   set g(state_tip_initial)      0
   set g(state_tip_regmanage)    0
   set g(state_tip_regnew)       0
   set g(state_tip_regclick)     0
   set g(videt_showing)          0
   set g(viglobe_showing)        0
   set g(viglobe_spin)           0
   set g(viglobe_render)         0
   set g(climpar_showing)        0
   set g(climpar_temp)         avg
   set g(climpar_rain)         avg
   set g(climpar_seasel)       avg
   set g(climpar_season)         0
   set g(climpar_interpol)       0
   set g(help_showing)           0
   set g(region_showing)         0
   set g(region_selected)       -1
   set g(region_lastsel)        -1 
   set g(region_highest)         0
   set g(region_height)          2
   set g(region_valid)           0
   set g(region_seqno)           0
   set g(genmineral_showing)     0
   util_flags }


proc init_menus { } {
   # Menu bar
   frame .menubar -relief raised -borderwidth 2

   # File menu
   menubutton .menubar.file -text "File" -underline 0 -menu .menubar.file.menu
   menu .menubar.file.menu -tearoff no
   .menubar.file.menu add command -label "New" -underline 0 \
      -command { file_new }
   .menubar.file.menu add command -label "Open..." -underline 0 \
      -command { file_open }
   .menubar.file.menu add command -label "Save..." -underline 0 \
      -command { file_save }
   .menubar.file.menu add cascade -label "Import" -underline 0 \
      -menu .menubar.file.menu.import
   .menubar.file.menu add cascade -label "Export" -underline 0 \
      -menu .menubar.file.menu.export
   .menubar.file.menu add command -label "Exit" -underline 1 -command { exit }

   # File export submenu
   menu .menubar.file.menu.export -tearoff no
   .menubar.file.menu.export add command -label "GIF..." -underline 0 \
      -command { file_export_gif }
   .menubar.file.menu.export add command -label "Land..." -underline 0 \
      -command { file_export_land }

   # File import submenu
   menu .menubar.file.menu.import -tearoff no
   .menubar.file.menu.import add command -label "Land..." -underline 0 \
      -command { file_import_land }

   # View menu
   menubutton .menubar.view -text "View" -underline 0 -menu .menubar.view.menu
   menu .menubar.view.menu -tearoff no
   .menubar.view.menu add command -label "Zoom in" -underline 0 \
      -command { view_zoomin }
   .menubar.view.menu add command -label "Zoom out" -underline 1 \
      -command { view_zoomout }
   .menubar.view.menu add command -label "Land" -underline 0 \
      -command { view_land }
   .menubar.view.menu add command -label "Climate" -underline 0 \
      -command { view_clim clim }
   .menubar.view.menu add command -label "Temperature" -underline 0 \
      -command { view_clim temp }
   .menubar.view.menu add command -label "Rainfall" -underline 0 \
      -command { view_clim rain }
   .menubar.view.menu add command -label "Globe..." -underline 0 \
      -command { view_globe }
   .menubar.view.menu add command -label "Details..." -underline 0 \
      -command { view_details }

   # Land menu
   menubutton .menubar.land -text "Land" -underline 0 \
      -menu .menubar.land.menu
   menu .menubar.land.menu -tearoff no
   .menubar.land.menu add command -label "Drift 1" -underline 0 \
      -command { land_n 1 }
   .menubar.land.menu add command -label "Drift 10" -underline 1 \
      -command { land_n 10 }
   .menubar.land.menu add command -label "Anchor" -underline 0 \
      -command { land_anchor }
   .menubar.land.menu add cascade -label "Edit" -underline 0 \
      -menu .menubar.land.menu.edit

   # Land edit submenu
   menu .menubar.land.menu.edit -tearoff no
   .menubar.land.menu.edit add command -label "Undo" -underline 1 \
      -command { land_undo }
   .menubar.land.menu.edit add command -label "Left" -underline 0 \
      -command { land_pan -1 0 }
   .menubar.land.menu.edit add command -label "Right" -underline 0 \
      -command { land_pan 1 0 }
   .menubar.land.menu.edit add command -label "Up" -underline 0 \
      -command { land_pan 0 -1 }
   .menubar.land.menu.edit add command -label "Down" -underline 0 \
      -command { land_pan 0 1 }
   .menubar.land.menu.edit add command -label "Flip" -underline 0 \
      -command { land_flip }
   .menubar.land.menu.edit add command -label "Sharpen" -underline 0 \
      -command { land_sharpen }
   .menubar.land.menu.edit add command -label "Crumble" -underline 0 \
      -command { land_crumble }

   # Climate menu
   menubutton .menubar.clim -text "Climate" -underline 0 \
      -menu .menubar.clim.menu
   menu .menubar.clim.menu -tearoff no
   .menubar.clim.menu add command -label "Compute" -underline 0 \
      -command { clim_compute }
   .menubar.clim.menu add command -label "Parameters..." -underline 0 \
      -command { clim_params }
   .menubar.clim.menu add command -label "Add rivers" -underline 0 \
      -command { clim_river }

   # Region menu
   menubutton .menubar.region -text "Region" -underline 0 \
      -menu .menubar.region.menu
   menu .menubar.region.menu -tearoff no
   .menubar.region.menu add command -label "Manage..." -underline 0 \
      -command { region_manage }
   .menubar.region.menu add command -label "Clear" -underline 0 \
      -command { region_clear }
   .menubar.region.menu add cascade -label "Grow" -underline 0 \
      -menu .menubar.region.menu.grow
   .menubar.region.menu add cascade -label "Generate" -underline 1 \
      -menu .menubar.region.menu.gen

   # Region grow submenu
   menu .menubar.region.menu.grow -tearoff no
   .menubar.region.menu.grow add command -label "Coast" -underline 0 \
      -command { region_grow coast }
   .menubar.region.menu.grow add command -label "Mountain" -underline 0 \
      -command { region_grow mountain }
   .menubar.region.menu.grow add command -label "Bounding box" -underline 0 \
      -command { region_grow box }
   .menubar.region.menu.grow add command -label "Climate" -underline 1 \
      -command { region_grow climate }

   # Region generate submenu
   menu .menubar.region.menu.gen -tearoff no
   .menubar.region.menu.gen add command -label "Animal..." -underline 0 \
      -command { gen_create animal }
   .menubar.region.menu.gen add command -label "Mineral..." -underline 0 \
      -command { gen_create mineral }
   .menubar.region.menu.gen add command -label "Vegetable..." -underline 0 \
      -command { gen_create vegetable }
   .menubar.region.menu.gen add command -label "Batch" -underline 0 \
      -command { gen_batch }

   # Help menu
   menubutton .menubar.help -text "Help" -underline 0 \
      -menu .menubar.help.menu
   menu .menubar.help.menu -tearoff no
   .menubar.help.menu add command -label "About..." -underline 0 \
      -command { help_about } }


proc initialize { } {
   global g argv0 tcl_platform

   init_vars
   init_menus

   # Source other files
   set dir [ file dirname $argv0 ]
   source  [ file join $dir splash.tcl ]
   source  [ file join $dir climpar.tcl ]
   source  [ file join $dir videt.tcl ]
   source  [ file join $dir region.tcl ]
   source  [ file join $dir generate.tcl ]

   # Load img library
   if { ! [ string compare $tcl_platform(platform) unix ] } {   
      set imglib ../lib/libimg1.1.so
   } else {
      set imglib img1180.dll
   }
   load [ file join $dir $imglib ] Img

   # Main display and scrolling
   canvas .main -yscrollcommand ".scrolly set" -xscrollcommand ".scrollx set" \
      -width 450 -height 450
   .main create image 0 0 -image main_photo -tags photo -anchor nw
   .main config -scrollregion "0 0 450 450"
   scrollbar .scrolly -orient v -command ".main yview"
   scrollbar .scrollx -orient h -command ".main xview"
   bind .main <ButtonRelease-1> { util_click %x %y }

   # Top level packing
   label .working -relief sunken -borderwidth 2 -textvariable g(working_message)
   pack .menubar.file .menubar.view .menubar.land .menubar.clim -side left
   if { ! [ string compare [ wbSec ] "Developer version" ] } {
      pack .menubar.region -side left }
   pack .menubar.help -side right
   grid .menubar -row 0 -column 0 -columnspan 2 -sticky ew
   grid .main    -row 1 -column 0 -sticky nsew
   grid .scrolly -row 1 -column 1 -sticky ns
   grid .scrollx -row 2 -column 0 -sticky ew
   grid .working -row 3 -column 0 -columnspan 2 -sticky ew
   grid columnconf . 0 -weight 1
   grid rowconf . 1 -weight 1

   # Last minute stuff
   wm title . "World Builder"
   util_tipbox initial "Use File->New to create a new world" }


initialize
